package com.bocloud.sms.booter.controller;

import com.bocloud.sms.entity.Admission;
import com.bocloud.sms.interfaces.AdmissionService;
import com.bocloud.sms.model.AdmissionModel;
import com.bocloud.sms.service.utils.AdmissionUtil;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * ip访问限制控制器
 *
 * @author wangyu
 * @version 4.2.1-SNAPSHOT
 * @since 2019/11/12
 */
@RestController
@RequestMapping("/v1/admissions")
@Tag(name = "IP限制管理")
public class AdmissionController {

    private final AdmissionService admissionService;
    private final AdmissionUtil admissionUtil;

    public AdmissionController(AdmissionService admissionService, AdmissionUtil admissionUtil) {
        this.admissionService = admissionService;
        this.admissionUtil = admissionUtil;
    }

    /**
     * 获取IP访问限制列表
     *
     * @param pager 分页参数
     * @return 分页数据
     */
    @Operation(summary = "查询IP访问限制列表")
    @GetMapping
    public GeneralResult<GridBean<Admission>> list(Pager pager) {
        return admissionService.list(pager);
    }

    /**
     * 添加IP访问限制
     *
     * @param admissionModel ip和类型
     * @return 是否添加成功
     */
    @Operation(summary = "创建IP访问限制")
    @PostMapping
    public Result create(@RequestBody AdmissionModel admissionModel,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        Assert.notNull(admissionModel, "请求参数不能为空");
        Result result = admissionService.create(admissionModel, requestContext);
        if (result.isSuccess()) {
            admissionUtil.refresh();
        }
        return result;
    }

    /**
     * 删除IP访问限制
     *
     * @param id 被删除的id
     * @return 是否删除成功
     */
    @Operation(summary = "删除IP访问限制")
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id) {
        Result result = admissionService.remove(id);
        if (result.isSuccess()) {
            admissionUtil.refresh();
        }
        return result;
    }

    /**
     * 查看IP访问限制详情
     *
     * @param id 被查看的数据id
     * @return id访问限制
     */
    @Operation(summary = "查看IP访问限制")
    @GetMapping("/{id}")
    public GeneralResult<Admission> detail(@PathVariable(value = Common.ID) Long id) {
        return admissionService.detail(id);
    }

    /**
     * 批量删除IP访问限制
     *
     * @param ids 被删除的id列表
     * @return 删除结果
     */
    @Operation(summary = "批量删除IP访问限制")
    @DeleteMapping
    public Result batchRemove(@RequestBody List<Long> ids) {
        return admissionService.batchRemove(ids);
    }

}
