package com.bocloud.sms.booter.controller;

import com.bocloud.sms.entity.Dictionary;
import com.bocloud.sms.interfaces.DictService;
import com.bocloud.sms.model.DictionaryBean;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Result;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 字典信息控制器
 *
 * @author dmw
 */
@RestController
@RequestMapping("/v1/dictionaries")
@Tag(name = "字典信息管理")
public class DictController {
    private final DictService dictService;

    public DictController(DictService dictService) {
        this.dictService = dictService;
    }

    @Operation(summary = "查询字典树列表")
    @GetMapping
    public GeneralResult<List<Dictionary>> list(@RequestParam(value = "pid", required = false) Long pid) {
        return dictService.list(pid);
    }

    @Operation(summary = "根据id查询字典")
    @GetMapping("/{id}")
    public GeneralResult<Dictionary> detail(@PathVariable(value = "id") Long id) {
        return dictService.detail(id);
    }

    /**
     * 根据字典值获取子节点列表
     *
     * @param value 字典值
     * @return 子节点列表
     */
    @Operation(summary = "查询系统字典子信息")
    @GetMapping("/children")
    public GeneralResult<List<Dictionary>> children(@RequestParam(value = Common.VALUE, required = false) String value,
                                                    @RequestParam(value = "pid", required = false) Long pid) {
        return dictService.listChild(value, pid);
    }

    /**
     * 查询字典树
     *
     * @param value 字典值
     * @return 字典树
     */
    @Operation(summary = "查询字典树")
    @GetMapping("/children/tree")
    public GeneralResult<List<Dictionary>> childTree(@RequestParam(value = Common.VALUE, required = false) String value) {
        return dictService.childTree(value);
    }

    /**
     * 创建
     *
     * @param dictionary 字典数据
     * @return 创建结果
     */
    @Operation(summary = "新增字典")
    @PostMapping
    public GeneralResult<Dictionary> create(@RequestBody Dictionary dictionary) {
        return dictService.create(dictionary);
    }

    /**
     * 删除字典
     *
     * @param id 字典id
     * @return 删除结果
     */
    @Operation(summary = "删除字典")
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id) {
        return dictService.remove(id);
    }

    /**
     * 修改字典
     *
     * @param id   字典id
     * @param bean 字典
     * @return 修改结果
     */
    @Operation(summary = "修改字典")
    @PutMapping("/{id}")
    public GeneralResult<Dictionary> modify(@PathVariable(value = Common.ID) Long id,
                                            @RequestBody DictionaryBean bean) {
        bean.setId(id);
        return dictService.modify(bean);
    }

}
