package com.bocloud.sms.booter.scheduler;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.sms.entity.AppEntity;
import com.bocloud.sms.enums.YkInf;
import com.bocloud.sms.service.utils.YkUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import java.util.*;

/**
 * 从运控获取应用列表
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class YunKongGetAppSchedule {
    private final Logger logger = LoggerFactory.getLogger(this.getClass());
    private final YkUtils ykUtils;
    private final StringRedisTemplate redisTemplate;


    @Scheduled(cron = "${yk.getAppListCron:0 0/30 * * * ?}")
    public void syncAppList() {
        logger.info("同步应用列表");
        try{
            JSONObject params = new JSONObject();
            params.put("type", "user");
            params.put("page", "0");
            params.put("limit", "20");

            JSONObject obj = ykUtils.call(YkInf.getAppList,params,JSONObject.class);
            List<AppEntity> appList = JSONArray.parseArray(obj.getString("list"),AppEntity.class);
            redisTemplate.opsForValue().set("SYS_APP_LIST", JSONObject.toJSONString(appList));
        }catch (Exception e){
            logger.error("同步应用列表异常",e);
        }
    }
}
