/**
 * @author hty
 * @time 上午9:39:37
 * @date 2014年12月5日
 */
package com.bocloud.sms.booter.websocket.message;

import com.auth0.jwt.interfaces.Claim;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.Tokens;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.server.ServerHttpRequest;
import org.springframework.http.server.ServerHttpResponse;
import org.springframework.http.server.ServletServerHttpRequest;
import org.springframework.http.server.ServletServerHttpResponse;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.socket.WebSocketHandler;
import org.springframework.web.socket.server.HandshakeInterceptor;

import java.util.Map;

@Component
@Slf4j
public class MessageInterceptor implements HandshakeInterceptor {

    @Override
    public boolean beforeHandshake(@NonNull ServerHttpRequest request, @NonNull ServerHttpResponse response, @NonNull WebSocketHandler wsHandler,
                                   @NonNull Map<String, Object> attributes) throws Exception {
        if (request instanceof ServletServerHttpRequest) {
            HttpServletRequest httpRequest = ((ServletServerHttpRequest) request).getServletRequest();
            String token = httpRequest.getHeader("Sec-WebSocket-Protocol");
            if (!StringUtils.hasText(token)) {
                return false;
            }
            Claim claim = Tokens.parse(token);
            if (null == claim) {
                return false;
            }
            String catalog = claim.asMap().get("catalog").toString();
            if (!StringUtils.hasText(catalog)) {
                return false;
            }
            Long id = Long.valueOf(claim.asMap().get(Common.UUID).toString());
            attributes.put("catalog", catalog);
            attributes.put("id", id);
            return true;
        }
        return false;
    }

    @Override
    public void afterHandshake(ServerHttpRequest request, ServerHttpResponse response, WebSocketHandler wsHandler,
                               Exception exception) {
        if (request instanceof ServletServerHttpRequest) {
            HttpServletRequest servletRequest = ((ServletServerHttpRequest) request).getServletRequest();
            HttpServletResponse servletResponse = ((ServletServerHttpResponse) response).getServletResponse();
            String SEC_WEBSOCKET_PROTOCOL = "Sec-WebSocket-Protocol";
            if (StringUtils.hasText(servletRequest.getHeader(SEC_WEBSOCKET_PROTOCOL))) {
                servletResponse.addHeader("Sec-WebSocket-Protocol", servletRequest.getHeader("Sec-WebSocket-Protocol"));
            }
        }
    }
}