package com.bocloud.sms.interfaces;

import com.bocloud.sms.entity.Dictionary;
import com.bocloud.sms.entity.Plugin;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Result;

import java.util.List;
import java.util.Map;

/**
 * 插件
 *
 * @author zhangyf
 * @since 2022-05-19
 */
public interface PluginService {

    /**
     * 查询列表
     *
     * @param pager 分野
     * @return plugin列表
     */
    GeneralResult<GridBean<Plugin>> list(Pager pager);

    /**
     * 已安装列表
     *
     * @param service 主服务
     * @return 插件列表
     */
    Map<String, List<Plugin>> listInstalled(String service);

    /**
     * 可更新列表
     *
     * @param service 主服务
     * @return 插件列表
     */
    Map<String, List<Plugin>> listUpdate(String service);

    /**
     * 可安装列表
     *
     * @param service 主服务
     * @return 插件列表
     */
    Map<String, List<Plugin>> listCanInstall(String service);


    /**
     * 添加
     *
     * @param plugin 插件
     * @return 成功/失败
     */
    Result create(Plugin plugin);

    /**
     * 更新
     *
     * @param id         ID
     * @param plugin     实体类
     * @param hasPackage 是否修改插件包
     * @return 更新 成功/失败
     */
    Result modify(Long id, Plugin plugin, boolean hasPackage);

    /**
     * 移除
     *
     * @param id
     * @return 删除
     */
    Result remove(Long id);

    /**
     * 批量删除
     *
     * @param idList
     * @return 成功/失败 个数
     */
    Result batchRemove(List<Long> idList);

    /**
     * 获取详细信息
     *
     * @param id
     * @return plugin详细信息
     */
    Plugin detail(Long id);

    /**
     * 查询插件下的云平台列表
     *
     * @param pager
     * @return 云平台列表 to ims
     */
    GeneralResult<List<Dictionary>> getVendorList(Pager pager);

    /**
     * 获取license中插件列表
     *
     * @param pager 分页参数
     * @return license中插件列表
     */
    List<Plugin> listLicense(Pager pager);

    /**
     * 当前运行的插件集合
     *
     * @param services 主服务
     * @return 插件集合
     */
    List<Plugin> runningPlugins(String... services);

}
