package com.bocloud.sms.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.sms.entity.User;
import com.bocloud.sms.enums.WarnInfo;
import com.bocloud.sms.enums.YkInfo;
import com.bocloud.sms.interfaces.IndexService;
import com.bocloud.sms.model.AppPerm;
import com.bocloud.sms.repository.UserRepository;
import com.bocloud.sms.service.utils.QxUtils;
import com.bocloud.sms.utils.WarnUtil;
import com.bocloud.sms.utils.YkUtil;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service("IndexService")
@Slf4j
public class IndexServiceImpl implements IndexService {

    @Autowired
    private YkUtil ykUtil;
    @Autowired
    private UserRepository userRepository;
    @Autowired
    private WarnUtil warnUtil;
    @Autowired
    private QxUtils qxUtils;

    @Override
    public GeneralResult doneWorkOrder(Pager pager, RequestContext requestContext) {
        try {
            JSONObject param = new JSONObject();
            User user = userRepository.query(requestContext.getTarget());
            log.info("user:{}", JSON.toJSONString(user));
            param.put("userId", user.getUserId() == null ? ykUtil.getReqUserId() : user.getUserId());
            param.put("page", pager.getPage());
            param.put("limit", pager.getRows());
            String result = ykUtil.call(YkInfo.getTaskList, param, JSONArray.class);
            JSONObject jsonObject = JSONArray.parseObject(result, JSONObject.class);
            return new GeneralResult(true, jsonObject, "待办工单列表查询成功");
        } catch (Exception e) {
            log.info("待办工单列表查询失败", e);
            return new GeneralResult(false, "待办工单列表查询失败");
        }
    }

    @Override
    public GeneralResult workOrderStatic(RequestContext requestContext) {
        try {

            JSONObject jsonObject = getTaskList(requestContext);
            Integer total = jsonObject.getInteger("total");

            Map<String, Object> resultMap = new HashMap<>();
            resultMap.put("orderTotal", total);

            List<JSONObject> data = JSONObject.parseObject(jsonObject.getString("data"), List.class);
            Integer waitDoneTotal = 0;
            Integer doneTotal = 0;
            for (JSONObject obj : data) {
                if ("0".equals(obj.getString("approve_status"))) {
                    waitDoneTotal++;
                } else {
                    doneTotal++;
                }
            }

            resultMap.put("waitDoneTotal", jsonObject.getInteger("todoCount"));
            resultMap.put("doneTotal", jsonObject.getInteger("doneCount"));

            return new GeneralResult(true, resultMap, "工单统计查询成功");
        } catch (Exception e) {
            log.info("工单统计查询失败", e);
            return new GeneralResult(false, "工单统计查询失败");
        }
    }

    private JSONObject getTaskList(RequestContext requestContext) throws Exception {
        JSONObject param = new JSONObject();
        User user = userRepository.query(requestContext.getTarget());
        log.info("user:{}", JSON.toJSONString(user));
        param.put("userId", user.getUserId() == null ? ykUtil.getReqUserId() : user.getUserId());
        param.put("page", 1);
        param.put("limit", 15); //理想要求 固定值15
        log.info("get task list params:{}", JSON.toJSONString(param));
        String result = ykUtil.call(YkInfo.getTaskList, param, JSONArray.class);
        log.info("result:{}",result);
        JSONObject jsonObject = JSONArray.parseObject(result, JSONObject.class);
        return jsonObject;
    }

    @Override
    public GeneralResult userDataStatic(RequestContext requestContext) {
        try {
            //获取用户数据权限申请数量
            JSONObject userParam = new JSONObject();
            User user = userRepository.query(requestContext.getTarget());
            log.info("user:{}", JSON.toJSONString(user));
            userParam.put("userId", user.getUserId() == null ? ykUtil.getReqUserId() : user.getUserId());
            String data = ykUtil.call(YkInfo.getCountTaskByUser, userParam, JSONArray.class);
            List<JSONObject> list = JSONObject.parseObject(data, List.class);


            return new GeneralResult(true, list, "获取用户数据权限申请数量查询成功");
        } catch (Exception e) {
            log.info("获取用户数据权限申请数量查询失败", e);
            return new GeneralResult(false, "获取用户数据权限申请数量查询失败");
        }
    }

    @Override
    public GeneralResult userAppList(RequestContext requestContext, String trxToken) {

        try {
            //获取所有的appList
            //查询当前用户授权应用
            JSONObject param = new JSONObject();
            param.put("type", "user");
            param.put("page", 1);
            param.put("limit", 15);
            String result = ykUtil.call(YkInfo.getAllAppList, param, JSONArray.class);
            JSONObject allAppJson = JSONObject.parseObject(result, JSONObject.class);
            List<JSONObject> allAppList = JSONObject.parseObject(allAppJson.getString("list"), List.class);
            log.info("allAppList:{}", JSON.toJSONString(allAppList));

//            List<JSONObject> userAppList = getUserAppList(requestContext);
//            log.info("userAppList:{}", JSON.toJSONString(userAppList));
//            List<String> appIdList = userAppList.stream().map(jsonObject -> jsonObject.getString("appId")).collect(Collectors.toList());

            // 调用权限系统接口
            User user = userRepository.query(requestContext.getTarget());
            log.info("user:{}", JSON.toJSONString(user));
            List<AppPerm> permList = JSONArray.parseArray(qxUtils.queryApp(trxToken, user.getUserId()), AppPerm.class);
            if (!CollectionUtils.isEmpty(permList)) {
                List<String> appIdList = permList.stream()
                        .map(AppPerm::getId)
                        .collect(Collectors.toList());
                log.info("appIdList:{}", JSON.toJSONString(appIdList));
                for (JSONObject data : allAppList) {
                    if (appIdList.contains(data.getString("appId"))) {
                        data.put("enable", "1");
                    } else {
                        data.put("enable", "0");
                    }
                }
            }
            log.info("allAppList:{}", JSON.toJSONString(allAppList));
            return new GeneralResult(true, allAppList, "首页数据展示模块查询成功");
        } catch (Exception e) {
            log.error("首页数据展示模块查询失败", e);
            return new GeneralResult(true,new ArrayList<>(),"首页数据展示模块查询失败");
        }
    }

    @Override
    public GeneralResult sevenDayWarnTrend(RequestContext requestContext) {

        try {
            String result = warnUtil.call(WarnInfo.get7DayWarnTrend.getUrl(), "GET");
            JSONObject jsonObject = JSONObject.parseObject(result);
            if (!"200".equals(jsonObject.getString("code"))) {
                log.info("result:" + result);
                return new GeneralResult(false, "近7日告警趋势查询失败");
            }
            JSONObject data = jsonObject.getJSONObject("data");
            return new GeneralResult(true, data, "近7日告警趋势查询成功");
        } catch (Exception e) {
            log.info("近7日告警趋势查询失败", e);
            return new GeneralResult(false, "近7日告警趋势查询失败");
        }
    }

    @Override
    public GeneralResult oneDayWarnCount(RequestContext requestContext) {

        try {
            String result = warnUtil.call(WarnInfo.get1DayWarnCount.getUrl(), "GET");
            JSONObject jsonObject = JSONObject.parseObject(result);
            if (!"200".equals(jsonObject.getString("code"))) {
                log.info("result:" + result);
                return new GeneralResult(false, "单日告警统计查询失败");
            }
            JSONObject data = jsonObject.getJSONObject("data");
            return new GeneralResult(true, data, "单日告警统计查询成功");
        } catch (Exception e) {
            log.info("单日告警统计查询失败", e);
            return new GeneralResult(false, "单日告警统计查询失败");
        }
    }

    @Override
    public GeneralResult warnList(RequestContext requestContext) {
        try {
            String result = warnUtil.call(WarnInfo.getWarnList.getUrl(), "GET");
            JSONObject jsonObject = JSONObject.parseObject(result);
            if (!"200".equals(jsonObject.getString("code"))) {
                log.info("result:" + result);
                return new GeneralResult(false, "单日告警统计查询失败");
            }
            List<JSONObject> data = JSONObject.parseObject(jsonObject.getString("data"), List.class);
            return new GeneralResult(true, data, "单日告警统计查询成功");
        } catch (Exception e) {
            log.info("单日告警统计查询失败", e);
            return new GeneralResult(false, "单日告警统计查询失败");
        }
    }

    private List<JSONObject> getUserAppList(RequestContext requestContext) throws Exception {
        //查询当前用户授权应用
        JSONObject param = new JSONObject();
        User user = userRepository.query(requestContext.getTarget());
        log.info("user:{}", JSON.toJSONString(user));
        param.put("userId", user.getUserId() == null ? ykUtil.getReqUserId() : user.getUserId());
        param.put("type", "user");
        log.info("get user app list params:{}", JSON.toJSONString(param));
        String result = ykUtil.call(YkInfo.getUserAppList, param, JSONArray.class);
        log.info("get user app list result:" + result);
        return JSONObject.parseObject(result, List.class);
    }
}
