package com.bocloud.sms.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.sms.entity.*;
import com.bocloud.sms.interfaces.SingleLoginService;
import com.bocloud.sms.interfaces.UserService;
import com.bocloud.sms.model.UserBean;
import com.bocloud.sms.model.single.*;
import com.bocloud.sms.repository.RoleRepository;
import com.bocloud.sms.repository.UserRepository;
import com.bocloud.sms.service.utils.HttpClientUtil;
import com.megatron.common.encrypt.AESEncryptor;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Result;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.NicTools;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.utils.IpTool;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.util.Asserts;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import org.springframework.web.servlet.view.RedirectView;

import java.io.IOException;
import java.net.URL;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

@Service("singleLoginService")
@Slf4j
public class SingleLoginServiceImpl implements SingleLoginService {

    private static final RequestContext CONTEXT = new RequestContext(RequestContext.Catalog.Manager, 1L, 0L, null, null);

    @Value("${single.system.url:https://127.0.0.1:80}")
    private String singleLoginUrl;

    @Value("${single.operation.control.url:https://127.0.0.1:90}")
    private String operationControlUrl;

    @Value("${single.operation.control.userId:112333}")
    private String reqUserId;

    @Value("${single.operation.control.systemId:1}")
    private String systemId;

    @Value("${single.cmcRedirectUri:https://10.20.12.88:60006}")
    private String singleCmcRedirectUri;

    @Value("${single.role.url:https://127.0.0.1:90}")
    private String roleUrl;

    @Value("${single.role.appId:https://127.0.0.1:90}")
    private String appId;

    @Value("${single.role.appName:https://127.0.0.1:90}")
    private String appName;

    @Value("${authing.role:超级管理员}")
    private String authingRole;

    @Autowired
    private UserRepository userRepository;
    @Autowired
    private UserService userService;
    @Autowired
    private RoleRepository roleRepository;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private CurrentService currentService;

    private String getRedisKey(long userId) {
        return "single_user_roles_"+ userId;
    }

    @Override
    public RedirectView singleLogin(String token, HttpServletRequest request) throws Exception {

        String userId = verifyToken(token, request);

        // UserResponseModel.User user = queryUser(userId);

        List<RoleResponseModel.Role> roles = queryUserRole(token, userId);

        User cmpUser = this.saveUser(userId, CONTEXT);
        log.info("cmpUser：" + JSON.toJSONString(cmpUser));

        Role cmpRole = roleRepository.exists(authingRole, 0L);
        Asserts.notNull(cmpRole, "未找到角色：" + authingRole);

        cmpUser.setIsManager(true);
        userRepository.update(cmpUser);

        Result accredit = userService.accredit(cmpUser.getId(), Collections.singletonList(cmpRole.getId()), CONTEXT);
        Asserts.check(accredit.isSuccess(), accredit.getMessage());
        log.info("set user[{}] as role[{}]", cmpUser.getName(), cmpRole.getName());

        GeneralResult<Map<String, Object>> loginResult = userService.login(cmpUser.getAccount(), getSecretPassword(),
                request.getSession().getId(), IpTool.getIP(request), true);
        Asserts.check(loginResult.isSuccess(), loginResult.getMessage());

        redisTemplate.opsForValue().set(getRedisKey(cmpUser.getId()), JSON.toJSONString(roles));

        URL url = new URL(singleCmcRedirectUri);
        String redirect = url.getProtocol() + "://" + url.getHost() + ":" + url.getPort() + "/#/sso?token=" + loginResult.getData().get(Common.TOKEN);
        return new RedirectView(redirect);
    }


    private String verifyToken(String token, HttpServletRequest request) {
        VerifyTokenRequestModel requestModel = new VerifyTokenRequestModel();
        requestModel.setToken(token);
        requestModel.setClientIp(getIP(request));

        String url = singleLoginUrl + "/verifyToken";
        String result = HttpClientUtil.invoke(
                JSON.toJSONString(requestModel),
                url
        );

        VerifyTokenResponseModel response = JSON.parseObject(result, VerifyTokenResponseModel.class);
        Assert.isTrue(response.getResult() == 0, "验证token失败：" + response.getErrmsg());

        Assert.notNull(response.getUserId(), "userId为空");
        return response.getUserId();
    }

    public String getIP(HttpServletRequest request) {
        String address = request.getHeader("X-Real_IP");
        String unknown = "unknown";
        if (StringUtils.isEmpty(address) || unknown.equalsIgnoreCase(address)) {
            address = request.getHeader("Proxy-Client-IP");
        }

        if (StringUtils.isEmpty(address) || unknown.equalsIgnoreCase(address)) {
            address = request.getHeader("WL-Proxy-Client-IP");
        }

        if (StringUtils.isEmpty(address) || unknown.equalsIgnoreCase(address)) {
            address = request.getRemoteAddr();
        }

        String defaultIpv6 = "0:0:0:0:0:0:0:1";
        if (defaultIpv6.equals(address)) {
            address = currentService.getService().getHost();
        }

        return address;
    }

    private UserResponseModel.User queryUser(String userId) throws IOException {
        String method = "/system/user/queryAllUser";

        UserRequestModel request = new UserRequestModel();
        request.setReqUserId(reqUserId);
        request.setSystemId(systemId);
        request.setMethod(method);

        UserRequestModel.Params params = new UserRequestModel.Params();
        params.setUserId(userId);
        request.setParams(params);


        String result = HttpClientUtil.invoke(JSON.toJSONString(request), operationControlUrl + method);

        UserResponseModel response = JSON.parseObject(result, UserResponseModel.class);
        Assert.isTrue(response.getReturnCode() == 1, response.getMsg());

        List<UserResponseModel.User> data = response.getData();
        Assert.isTrue(!ListTool.isEmpty(data), String.format("根据userId：%s未找到用户信息", userId));

        return data.get(0);
    }


    private List<RoleResponseModel.Role> queryUserRole(String token, String userId) {
        String method = "/api/v1/appModules";

        RoleRequestModel request = new RoleRequestModel();
        request.setUserToken(token);
        request.setUserId(userId);
        request.setAppId(appId);
        request.setAppName(appName);

        String result = HttpClientUtil.invoke(JSON.toJSONString(request), roleUrl + method);

        RoleResponseModel response = JSON.parseObject(result, RoleResponseModel.class);
        Assert.isTrue(response.getStatus().equals("200"), response.getMsg());

        List<RoleResponseModel.Role> data = response.getData();
        if (ListTool.isEmpty(data)) {
            return new ArrayList<>();
        }
        data.forEach(
                role -> {
                    if (StringUtils.isEmpty(role.getPid())) {
                        role.setPid("0");
                    }
                    if (role.getPid().equals(role.getId())) {
                        role.setPid("0");
                    }
                }
        );
        return treeRole("0", data);
    }

    public static List<RoleResponseModel.Role> treeRole(String parentId, List<RoleResponseModel.Role> roles) {
        List<RoleResponseModel.Role> list = new ArrayList<>();
        // 第一次筛选出主父节点列表进入循环，循环里面 进入递归 筛选出递归传递的从父节点列表
        // 递归，最末的父节点从整个列表筛选出它的子节点列表依次组装
        for (RoleResponseModel.Role root : roles) {
            if (root.getPid().equals(parentId)) {
                List<RoleResponseModel.Role> children = treeRole(root.getId(), roles);
                root.setChildren(children);
                list.add(root);
            }
        }
        return list;
    }



    public User saveUser(String userId, RequestContext context) {
        User userEntity = userRepository.queryByUserId(userId);
        if (userEntity == null) {
            UserBean userBean = new UserBean();
            this.convertUser(userId, userBean);
            userBean.setIsManager(false);
            Result result = userService.create(userBean, context);
            Asserts.check(result.isSuccess(), result.getMessage());
            userEntity = userRepository.queryByUserId(userId);
        }
        return userEntity;
    }


    private void convertUser(String userId, UserBean userBean) {
        userBean.setUserId(userId);
        userBean.setName(userId);
        userBean.setAccount(userId);
        userBean.setPassword(getSecretPassword());

        userBean.setSex(false);

        userBean.setEmail(RandomStringUtils.randomAlphabetic(8) + "@qq.com");

        userBean.setMobile("138" + RandomStringUtils.randomNumeric(8));

        log.info("userBean：" + JSON.toJSONString(userBean));
    }

    private String getSecretPassword() {
        String plainPassword = "Cmp@v587";
        return new AESEncryptor().encrypt(plainPassword, null);
    }

}

