package com.bocloud.sms.service.internel;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BoCloudService;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.megatron.common.enums.PostDataFormat;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Cmp内部访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年1月15日
 */
@RequiredArgsConstructor
@Component
public class CmpInternalService {

    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;
    private static final Service SERVICE = Service.create(BoCloudService.CMP.getName());

    public GeneralResult detailPool(Long poolId) {
        Map<String, Object> headers = new HashMap<>(4);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/pool/groups/" + poolId,
                HttpMethod.GET, headers, null);
        return remoteService.invoke();
    }

    public GeneralResult tenantCheck(List<Long> poolIds, Long tenantId) {
        JSONObject object = new JSONObject();
        object.put(Common.CONDITION, "tenantCheck");
        object.put("poolIds", poolIds);
        object.put("tenantId", tenantId);
        Map<String, Object> headers = new HashMap<>(4);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/pool/groups",
                HttpMethod.GET, headers, MapTools.simpleMap(Common.CONDITION, object.toJSONString()));
        return remoteService.invoke();
    }

    public GeneralResult createOpProject(List<Long> poolIds, Long tenantId, String apiKey) {
        JSONObject object = new JSONObject();
        object.put("poolIds", poolIds);
        object.put("tenantId", tenantId);
        Map<String, Object> headers = new HashMap<>(4);
        headers.put(Common.API_KEY, apiKey);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        RemoteService remoteService = serviceFactory.build(SERVICE, "/plugins/openstack/v1/ostenants/projects",
                HttpMethod.POST, headers, object);
        remoteService.setDataFormat(PostDataFormat.RAW);
        return remoteService.invoke();
    }

}
