package com.bocloud.sms.service.ldap;

import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

import javax.naming.Context;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;
import java.util.Hashtable;

@Data
@AllArgsConstructor
@NoArgsConstructor
public class LdapConnector {

    private String url;
    private String account;
    private String password;
    private String baseDn;
    private DirContext connector;
    private boolean accessible;

    public LdapConnector(String url, String account, String password, String baseDn) {
        super();
        this.url = url;
        this.account = account;
        this.password = password;
        this.baseDn = baseDn;
    }

    public void start() {
        Hashtable<String, String> env = new Hashtable<>();
        env.put(Context.SECURITY_AUTHENTICATION, "simple");
        env.put(Context.INITIAL_CONTEXT_FACTORY, "com.sun.jndi.ldap.LdapCtxFactory");
        env.put(Context.PROVIDER_URL, url);
        env.put(Context.SECURITY_PRINCIPAL, account);
        env.put(Context.SECURITY_CREDENTIALS, password);
        try {
            connector = new InitialDirContext(env);
            accessible = true;
        } catch (Exception e) {
            accessible = false;
        }
    }

    public void close() {
        if (null != connector) {
            try {
                connector.close();
            } catch (Exception e) {
            }
        }
    }

    public boolean accessable() {
        return this.accessible;
    }

    public String getBaseDn() {
        return baseDn;
    }

    public void setBaseDn(String baseDn) {
        this.baseDn = baseDn;
    }

    public DirContext getConnector() {
        return connector;
    }
}
