package com.bocloud.sms.service.monitor;


import com.bocloud.sms.monitor.LabelParam;

import java.util.List;
import java.util.Map;

/**
 * prometheus监控工具类
 *
 * @author zhangyf
 * @since 2018-12-27
 */
public class PrometheusUtils {

    private PrometheusUtils() {
    }

    /**
     * @param expr
     * @param params
     * @return
     */
    public static String putParams(String expr, Map<String, String> params) {
        if (params == null || params.isEmpty()) {
            return expr;
        }
        String endParam = getEndParam(expr);
        if (endParam != null && endParam.contains("{")) {
            endParam = putParams(endParam, params);
        }
        if (expr.contains("{")) {
            expr = expr.substring(0, expr.indexOf('{'));
        }
        StringBuilder builder = new StringBuilder();
        builder.append("{");
        params.forEach((key, value) -> builder.append(",").append(key).append("=").append("'").append(value).append("'"));
        expr += builder.toString().replaceFirst(",", "");

        if (null != endParam) {
            expr += endParam;
        } else {
            expr += '}';
        }
        return expr;
    }

    /**
     * @param expr
     * @param params
     * @return
     */
    public static String putParams(String expr, List<LabelParam> params) {
        if (params == null || params.isEmpty()) {
            return expr;
        }
        String endParam = getEndParam(expr);
        if (endParam != null && endParam.contains("{")) {
            endParam = putParams(endParam, params);
        }
        if (expr.contains("{")) {
            expr = expr.substring(0, expr.indexOf('{'));
        }
        StringBuilder builder = new StringBuilder();
        builder.append("{");
        params.forEach(param -> builder.append(",").append(param.getLabel()).append(param.getOperator().getOperator()).append("'").append(param.getValue()).append("'"));
        expr += builder.toString().replaceFirst(",", "");

        if (null != endParam) {
            expr += endParam;
        } else {
            expr += '}';
        }
        return expr;
    }


    /**
     * 截取尾部参数 即表达示“{”后的参数
     *
     * @param expr
     * @return
     */
    private static String getEndParam(String expr) {
        if (!expr.contains("{")) {
            return null;
        }
        String endParam = expr.substring(expr.indexOf('{') + 1);
        return "," + endParam;
    }
}
