package com.bocloud.sms.service.monitor;


import com.bocloud.sms.interfaces.PrometheusService;
import com.bocloud.sms.interfaces.QueryService;
import com.bocloud.sms.model.LineChartResult;
import com.bocloud.sms.monitor.PrometheusResult;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.*;

/**
 * prometheus监控图表
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0
 */
@Service
@Slf4j
public class QueryServiceImpl implements QueryService {


    @Autowired
    private PrometheusService prometheusService;

    /**
     * 监控折线图：时间-值
     * @param lineChartParam
     * @return
     */
    private LineChartResult lineChart(LineChartResult.LineChartParam lineChartParam) {
        List<PrometheusResult> list = prometheusService.queryRange(lineChartParam.getExpr(), lineChartParam.getStart(),
                lineChartParam.getEnd(), lineChartParam.getStep());
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        List<String> keys = new ArrayList<>();
        List<LineChartResult.Values> valuesList = new ArrayList<>();
        list = Optional.ofNullable(list).orElse(new ArrayList<>());
        list.forEach(prometheusResult -> {
            String name = lineChartParam.getName();
            if (lineChartParam.getName().indexOf("{") != -1) {
                String paramName = lineChartParam.getName().substring(lineChartParam.getName().indexOf("{") + 1, lineChartParam.getName().indexOf("}"));
                Object labelName = prometheusResult.getMetric().get(paramName.trim());
                if (labelName != null) {
                    name = lineChartParam.getName().replace("{" + paramName + "}", String.valueOf(labelName));
                }
            } else if (StringUtils.startsWith(lineChartParam.getName(), "$")) {
                Object labelName = prometheusResult.getMetric().get(lineChartParam.getName().substring(1));
                if (labelName != null) {
                    name = String.valueOf(labelName);
                }
            }
            List<Object> data = new ArrayList<>();
            for (Object[] value : prometheusResult.getValues()) {
                if (valuesList.isEmpty()) {
                    keys.add(sdf.format(new Date(Integer.parseInt(String.valueOf(value[0])) * 1000L)));
                }
                data.add(String.valueOf(value[1]));
            }
            valuesList.add(new LineChartResult.Values(name, data));
        });
        return new LineChartResult(keys, valuesList.toArray(new LineChartResult.Values[valuesList.size()]), null);
    }

    @Override
    public LineChartResult chart(List<String> exprs, List<String> names, String start, String end, String step, String unit) {
        LineChartResult result = null;
        for (int j = 0; j < exprs.size(); j++) {
            LineChartResult.LineChartParam param = new LineChartResult.LineChartParam();
            param.setName(names.get(j));
            param.setExpr(exprs.get(j));
            param.setStart(start);
            param.setEnd(end);
            param.setStep(step);
            LineChartResult lineChartResult = lineChart(param);
            if (result == null) {
                result = lineChartResult;
                result.setUnit(unit);
                continue;
            }
            LineChartResult.Values[] values = result.getValues();
            LineChartResult.Values[] newValues = new LineChartResult.Values[lineChartResult.getValues().length + values.length];
            for (int i = 0; i < values.length; i++) {
                newValues[i] = values[i];
            }
            for (int i = 0; i < lineChartResult.getValues().length; i++) {
                newValues[values.length + i] = lineChartResult.getValues()[i];
            }
            result.setValues(newValues);
        }
        return result;
    }



}
