package com.bocloud.sms.service.monitor;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.sms.monitor.ZooKeeper;
import lombok.extern.slf4j.Slf4j;
import org.apache.zookeeper.client.FourLetterWordMain;
import org.apache.zookeeper.common.X509Exception;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * zk监控类
 *
 * @author tyl
 * @since 2019/11/19 周二 11:54
 */
@Slf4j
public class ZooKeeperMonitor {

    /**
     * 获取zk节点状态信息
     *
     * @param host zk节点ip
     * @param port zk节点端口
     * @return zk节点状态信息
     */

    public static ZooKeeper getZooKeeperMonitor(String host, int port) {
        ZooKeeper zooKeeper = new ZooKeeper();
        String monitorString;
        try {
            monitorString = FourLetterWordMain.send4LetterWord(host, port, "mntr", false, 1000);
        } catch (IOException | X509Exception.SSLContextException e) {
            log.error("get zookeeper monitor info error：", e);
            zooKeeper.setHostPort(host + ":" + port);
            zooKeeper.setStatus("error");
            return zooKeeper;
        }
        String[] monitorData = monitorString.split("\n");
        Map<String, String> monitorEntry = new HashMap<>(16, 0.75F);
        for (String data : monitorData) {
            if (data.contains("\t")) {
                String key = data.substring(0, data.lastIndexOf("\t")).trim();
                String value = data.substring(data.indexOf("\t")).trim();
                monitorEntry.put(key, value);
            }
        }
        if (monitorEntry.size() > 0) {
            zooKeeper.setHostPort(host + ":" + port);
            zooKeeper.setStatus("running");
            zooKeeper.setMonitorInfo(JSONObject.parseObject(JSONObject.toJSONString(monitorEntry)));
        } else {
            log.error("zookeeper get 4LetterWord back error.");
            zooKeeper.setHostPort(host + ":" + port);
            zooKeeper.setStatus("error");
        }
        return zooKeeper;
    }
}
