package com.bocloud.sms.service.utils;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.megatron.common.model.Result;
import lombok.extern.slf4j.Slf4j;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.function.BiFunction;

/**
 * @Author hya
 * @Date 2022/3/9 10:10
 */
@Slf4j
public class ImportExcelUtil {
    private static Result result = null;

    /**
     * 获取读数据监听器
     *
     * @param biFunction 读取到数据后执行业务逻辑
     * @param threshold  数据量大时 在 invoke 方法里面多次读取时的边界
     * @param checked    表头，用来做模板检验
     * @param params     需要传递的其他参数
     * @return AnalysisEventListener
     */
    public static <T> AnalysisEventListener<T> getListener(BiFunction<List<T>, Map<String, Object>, Result> biFunction, int threshold, String[] checked, Map<String, Object> params) {
        LinkedList<T> list = new LinkedList<>();
        return new AnalysisEventListener<T>() {
            @Override
            public void invokeHeadMap(Map<Integer, String> headMap, AnalysisContext context) {
                super.invokeHeadMap(headMap, context);
                for (int i = 0; i < checked.length; i++) {
                    if (!headMap.get(i).equals(checked[i])) {
                        log.error("导入模板错误");
                        result = new Result(false, "导入模板错误");
                        throw new IllegalArgumentException("导入模板错误");
                    }
                }
            }

            @Override
            public void invoke(T data, AnalysisContext context) {
                list.add(data);
            }

            @Override
            public void doAfterAllAnalysed(AnalysisContext context) {
                if (list.size() > 0) {
                    result = biFunction.apply(list, params);
                }
            }
        };
    }

    public static Result getResult() {
        return result;
    }

}
