package com.bocloud.sms.entity;

import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.gitee.starblues.core.PluginState;
import com.megatron.common.utils.DateSerializer;
import com.megatron.entity.annotations.*;
import com.megatron.entity.bean.Generic;
import com.megatron.entity.meta.PKStrategy;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;

/**
 * 插件
 *
 * @author zhangyf
 * @since 2022-05-19
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@EqualsAndHashCode(callSuper = true)
@Table("plugin")
public class Plugin extends Generic {
    /**
     * 唯一ID
     */
    @PK(PKStrategy.AUTO)
    private Long id;

    /**
     * 插件名称
     */
    @Column("name")
    private String name;
    /**
     * 所属服务
     */
    @Column("service")
    private String service;

    /**
     * 插件编码
     */
    @Column("code")
    private String code;

    /**
     * 插件平台类型
     */
    @Column("vendor_type")
    private String vendorType;

    /**
     * 插件描述
     */
    @Column("remark")
    private String remark;

    /**
     * 是否删除
     */
    @Column("is_deleted")
    private Boolean deleted;

    /**
     * 插件状态true开启/false关闭
     */
    @Column("status")
    private Boolean status;

    /**
     * 创建时间
     */
    @Column("gmt_create")
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtCreate;

    /**
     * 修改时间
     */
    @Column("gmt_modify")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtModify;

    /**
     * 版本
     */
    @Column("version")
    private String version;

    /**
     * 插件路径
     */
    @Column("path")
    private String path;

    /**
     * 提供者
     */
    @Column("provider")
    private String provider;

    /**
     * 插件包大小
     */
    @Column("size")
    private Long size;

    /**
     * 插件状态
     */
    @Ignore
    private PluginState pluginState;
    /**
     * 所属主服务
     */
    @Ignore
    private String address;
    /**
     * 是否可更新
     */
    @Ignore
    private Boolean canUpdate;

    public Plugin(String code, Boolean status) {
        this.code = code;
        this.status = status;
    }

}
