package com.bocloud.sms.entity;

import com.megatron.entity.annotations.Column;
import com.megatron.entity.annotations.IgnoreAll;
import com.megatron.entity.annotations.PK;
import com.megatron.entity.annotations.Table;
import com.megatron.entity.bean.GenericEntity;
import com.megatron.entity.meta.PKStrategy;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;

import java.util.Date;

/**
 * 系统升级任务
 *
 * @author luyanchao
 * @date 2021-06-15
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@EqualsAndHashCode(callSuper = true)
@Table("upgrade_task")
public class UpgradeTask extends GenericEntity {
    /**
     * id
     */
    @PK(value = PKStrategy.AUTO)
    private Long id;
    /**
     * 所属升级历史id
     */
    @Column("upgrade_history_id")
    private Long upgradeHistoryId;
    /**
     * Spring中该Task的beanId
     */
    @Column("bean_id")
    private String beanId;
    /**
     * Java中该Task的类名称
     */
    @Column("class_name")
    private String className;
    /**
     * 执行结果
     */
    @Column("result")
    private String result;
    /**
     * 开始时间
     */
    @Column("start_time")
    private Date startTime;
    /**
     * 结束时间
     */
    @Column("end_time")
    private Date endTime;

    /**
     * 执行进度，该数据不入库，实时获取
     */
    @IgnoreAll
    private Integer percent;

    public Long getUsed() {
        if (startTime == null) {
            return null;
        }
        if (endTime == null) {
            return System.currentTimeMillis() - startTime.getTime();
        }
        return endTime.getTime() - startTime.getTime();
    }
}
