package com.bocloud.sms.repository;

import com.bocloud.sms.model.CloudServiceBean;
import com.megatron.common.model.License;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.license.LicenseDaemon;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Arrays;
import java.util.List;
import java.util.Map;


/**
 * 服务
 *
 * @author hya
 * @since 2022-03-02
 */
@Repository
public class CloudServiceBeanRepository extends BasicGenericDao<CloudServiceBean, Long> {

    @Value("${spring.datasource.cos:cos}")
    private String cosDb;

    private final CurrentService currentService;

    @Autowired
    public CloudServiceBeanRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService currentService) {
        super(jdbcTemplate, npJdbcTemplate, currentService);
        this.currentService = currentService;
    }

    /**
     * 获取收藏服务
     *
     * @param userId   用户ID
     * @param category 服务目录
     * @return 服务列表
     */
    public List<CloudServiceBean> getFavoriteServices(Long userId, String category) {
        License content = LicenseDaemon.instance(currentService).content();
        String sql = "select cs.*,sc.name catalog_name,i.icon icon,f.priority priority,f.id favorite_id from " + cosDb + ".cloud_service cs " +
                " left join " + cosDb + ".service_catalog sc on cs.catalog_id=sc.id " +
                " left join icon i on cs.icon_id = i.id " +
                " left join favorite f on cs.id = f.service_id " +
                " where cs.is_deleted = 0 and f.category = :category and f.target = :userId and cs.code not in (:excludeServiceCodes) and cs.status= 'PUBLISHED' " +
                " and (cs.plugin_code in (:pluginCode) or cs.plugin_code not in (:globals) or cs.plugin_code = '' or cs.plugin_code is NULL )";
        Map<String, Object> paramsMap = MapTools.simpleMap("userId", userId);
        paramsMap.put("category", category);
        paramsMap.put("globals", content.getGlobals());
        paramsMap.put("pluginCode", content.getPlugins());
        paramsMap.put("excludeServiceCodes", Arrays.asList("vmware.storage.disk", "cnware.standard.volume"));
        return super.list(CloudServiceBean.class, sql, paramsMap);
    }

    /**
     * 获取服务菜单
     *
     * @return 服务菜单
     */
    public List<CloudServiceBean> getServiceMenu() {
        License content = LicenseDaemon.instance(currentService).content();
        String sql = " select cs.*,sc.name catalog_name,i.icon icon from " + cosDb + ".cloud_service cs " +
                " left join " + cosDb + ".service_catalog sc on cs.catalog_id=sc.id " +
                " left join icon i on cs.icon_id = i.id " +
                " where cs.is_deleted = 0 and cs.status='PUBLISHED' and cs.code not in (:excludeServiceCodes) " +
                " and (cs.plugin_code in (:pluginCode) or cs.plugin_code not in (:globals) or cs.plugin_code = '' or cs.plugin_code is NULL )";
        Map<String, Object> map = MapTools.simpleMap("pluginCode", content.getPlugins());
        map.put("globals", content.getGlobals());
        map.put("excludeServiceCodes", Arrays.asList("vmware.storage.disk", "cnware.standard.volume"));
        return super.list(CloudServiceBean.class, sql, map);
    }

}
